@@ -125,8 +125,9 @@ prepare_crypto (GckSession *self, CK_MECHANISM_PTR mech,
 	CK_ULONG n_mechanisms, i;
 	gsize n_data;
 	gboolean have;
+	gulong key_type;
 	CK_RV rv;
-	
+
 	g_assert (GCK_IS_SESSION (self));
 
 	/* Cancel any current operation */
@@ -142,7 +143,8 @@ prepare_crypto (GckSession *self, CK_MECHANISM_PTR mech,
 	if (rv != CKR_OK)
 		return rv;
 
-	if (!GCK_IS_OBJECT (object))
+	/* Make sure it's a key */
+	if (!gck_object_get_attribute_ulong (object, self, CKA_KEY_TYPE, &key_type))
 		return CKR_KEY_HANDLE_INVALID;
 
 	/* Lookup the mechanisms this object can do */
@@ -756,62 +758,11 @@ gck_session_destroy_session_object (GckSession *self, GckTransaction *transactio
 	remove_object (self, transaction, obj);
 }
 
-gboolean
-gck_session_for_each_credential (GckSession *self, GckObject *object,
-                                 GckCredentialFunc func, gpointer user_data)
+GckCredential*
+gck_session_get_credential (GckSession *self)
 {
-	CK_OBJECT_HANDLE handle;
-	CK_OBJECT_CLASS klass;
-	CK_ATTRIBUTE attrs[2];
-	GList *results, *l;
-
-	g_return_val_if_fail (GCK_IS_SESSION (self), FALSE);
-	g_return_val_if_fail (GCK_IS_OBJECT (object), FALSE);
-	g_return_val_if_fail (func, FALSE);
-
-	/* Do we have one right on the session */
-	if (self->pv->credential != NULL &&
-	    gck_credential_get_object (self->pv->credential) == object) {
-		if ((func) (self->pv->credential, object, user_data))
-			return TRUE;
-	}
-
-	klass = CKO_G_CREDENTIAL;
-	attrs[0].type = CKA_CLASS;
-	attrs[0].pValue = &klass;
-	attrs[0].ulValueLen = sizeof (klass);
-
-	handle = gck_object_get_handle (object);
-	attrs[1].type = CKA_G_OBJECT;
-	attrs[1].pValue = &handle;
-	attrs[1].ulValueLen = sizeof (handle);
-
-	/* Find any on the session */
-	results = gck_manager_find_by_attributes (self->pv->manager,
-	                                          attrs, G_N_ELEMENTS (attrs));
-
-	for (l = results; l; l = g_list_next (l)) {
-		if ((func) (l->data, object, user_data))
-			break;
-	}
-
-	g_list_free (results);
-
-	if (l != NULL)
-		return TRUE;
-
-	/* Find any in the token */
-	results = gck_manager_find_by_attributes (gck_module_get_manager (self->pv->module), 
-	                                          attrs, G_N_ELEMENTS (attrs));
-
-	for (l = results; l; l = g_list_next (l)) {
-		if ((func) (l->data, object, user_data))
-			break;
-	}
-
-	g_list_free (results);
-	
-	return (l != NULL);
+	g_return_val_if_fail (GCK_IS_SESSION (self), NULL);
+	return self->pv->credential;
 }
 
 GckObject*
@@ -821,7 +772,6 @@ gck_session_create_object_for_factory (GckSession *self, GckFactory *factory,
 {
 	GckTransaction *owned = NULL;
 	GckObject  *object;
-	gulong i;
 
 	g_return_val_if_fail (GCK_IS_SESSION (self), NULL);
 	g_return_val_if_fail (factory && factory->func, NULL);
@@ -848,13 +798,6 @@ gck_session_create_object_for_factory (GckSession *self, GckFactory *factory,
 		gck_transaction_fail (transaction, CKR_GENERAL_ERROR);
 	}
 
-	/* Next go through and set all attributes that weren't used initially */
-	gck_attributes_consume (template, count, CKA_TOKEN, G_MAXULONG);
-	for (i = 0; i < count && !gck_transaction_get_failed (transaction); ++i) {
-		if (!gck_attribute_consumed (&template[i]))
-			gck_object_set_attribute (object, self, transaction, &template[i]);
-	}
-
 	g_free (template);
 
 	if (owned)
@@ -897,6 +840,7 @@ gck_session_complete_object_creation (GckSession *self, GckTransaction *transact
                                       GckObject *object, CK_ATTRIBUTE_PTR attrs, CK_ULONG n_attrs)
 {
 	gboolean is_private;
+	gulong i;
 
 	g_return_if_fail (GCK_IS_SESSION (self));
 	g_return_if_fail (GCK_IS_OBJECT (object));
@@ -910,24 +854,44 @@ gck_session_complete_object_creation (GckSession *self, GckTransaction *transact
 	/* See if we can create due to read-only */
 	if (gck_object_is_token (object)) {
 		if (!gck_object_is_transient (object) &&
-		    gck_module_get_write_protected (self->pv->module))
-			return gck_transaction_fail (transaction, CKR_TOKEN_WRITE_PROTECTED);
-		else if (self->pv->read_only)
-			return gck_transaction_fail (transaction, CKR_SESSION_READ_ONLY);
+		    gck_module_get_write_protected (self->pv->module)) {
+			gck_transaction_fail (transaction, CKR_TOKEN_WRITE_PROTECTED);
+			return;
+		}
+		else if (self->pv->read_only) {
+			gck_transaction_fail (transaction, CKR_SESSION_READ_ONLY);
+			return;
+		}
 	}
 
 	/* Can only create public objects unless logged in */
 	if (gck_session_get_logged_in (self) != CKU_USER &&
 	    gck_object_get_attribute_boolean (object, self, CKA_PRIVATE, &is_private) &&
 	    is_private == TRUE) {
-		return gck_transaction_fail (transaction, CKR_USER_NOT_LOGGED_IN);
+		gck_transaction_fail (transaction, CKR_USER_NOT_LOGGED_IN);
+		return;
 	}
 
-	/* Find somewhere to store the object */
-	if (gck_object_is_token (object))
-		gck_module_store_token_object (self->pv->module, transaction, object);
-	else
-		add_object (self, transaction, object);
+	/* Add the object to session or token */
+	if (!gck_transaction_get_failed (transaction)) {
+		if (gck_object_is_token (object))
+			gck_module_add_token_object (self->pv->module, transaction, object);
+		else
+			add_object (self, transaction, object);
+	}
+
+	/* Next go through and set all attributes that weren't used initially */
+	gck_attributes_consume (attrs, n_attrs, CKA_TOKEN, G_MAXULONG);
+	for (i = 0; i < n_attrs && !gck_transaction_get_failed (transaction); ++i) {
+		if (!gck_attribute_consumed (&attrs[i]))
+			gck_object_set_attribute (object, self, transaction, &attrs[i]);
+	}
+
+	/* Store the object */
+	if (!gck_transaction_get_failed (transaction)) {
+		if (gck_object_is_token (object))
+			gck_module_store_token_object (self->pv->module, transaction, object);
+	}
 }
 
 /* -----------------------------------------------------------------------------
@@ -1098,6 +1062,10 @@ gck_session_C_SetAttributeValue (GckSession* self, CK_OBJECT_HANDLE handle,
 	for (i = 0; i < count && !gck_transaction_get_failed (transaction); ++i) 
 		gck_object_set_attribute (object, self, transaction, &template[i]);
 
+	/* Store the object */
+	if (!gck_transaction_get_failed (transaction) && gck_object_is_token (object))
+		gck_module_store_token_object (self->pv->module, transaction, object);
+
 	gck_transaction_complete (transaction);
 	rv = gck_transaction_get_result (transaction);
 	g_object_unref (transaction);
